from ..core.requests import request_endpoint

async def chat(**kwargs):
    result = await request_endpoint("chat", **kwargs)
    return result

async def models_get(**kwargs):
    result = await request_endpoint("models_get", **kwargs)
    return result

async def model_delete(**kwargs):
    result = await request_endpoint("model_delete", **kwargs)
    return result

async def model_patch(**kwargs):
    result = await request_endpoint("model_patch", **kwargs)
    return result

async def model_post(**kwargs):
    """
    Creates a new model in the FalconAI service.

    Args:
        model_name (str): Name of the model.
        model_type (str): Type of the model, could be 'base'.
        version (str): Version of the model.
        year (int): Year of the model release.
        dataset_size (str): Size of the dataset used.
        provider (str): Provider of the model.
        modality (str): Modality of the model.
        max_tokens (str): Maximum tokens that the model can handle.
        description (str): Description of the model.
        model_attributes (str): Attributes of the model.

    Returns:
        The response from the FalconAI service.
    """
    
    result = await request_endpoint("model_post", **kwargs)
    return result


import httpx
from ..config import config

async def request_endpoint(endpoint_name, **kwargs):
    endpoint = config.get_endpoint_data(endpoint_name)
    if not endpoint:
        raise ValueError(f"No configuration found for endpoint: {endpoint_name}")

    method = endpoint['method']
    url = endpoint['url']
    headers = endpoint['headers']
    param_definitions = endpoint['params']
    data = extract_fields(endpoint["body"], **kwargs)
    
    # Replace path parameters in URL
    for key, value in kwargs.items():
        placeholder = f"{{{key}}}"
        if placeholder in url:
            url = url.replace(placeholder, str(value))
    
     # Build params based on what's actually passed in kwargs
    params = {param: kwargs[param] for param in param_definitions if param in kwargs}
        
    response = await make_request(method, url, headers, data, params)

    if response.get("error"):
        # Handle error
        print(f"Error: {response['error']}")
    else:
        # Process successful response
        data = response.get("data")
        
    return response

async def make_request(method, url, headers=None, data=None, params=None, ):
    try:
            
        async with httpx.AsyncClient() as client:
            response = await client.request(method, url, headers=headers, json=data, params=params)

            # Check if the response is successful
            if response.status_code >= 400:
                # Return error information as a dictionary
                return {
                    "status_code": response.status_code,
                    "error": response.json() if response.content else response.text
                }

            # Return successful response
            return {"status_code": response.status_code, "data": response.json()}

    except httpx.RequestError as e:
        print(f"An error occurred while requesting {e.request.url!r}.")
        return {"status_code": 0, "error": str(e)}

    except Exception as e:
        print(f"An unexpected error occurred: {e}")
        return {"status_code": 0, "error": str(e)}

def extract_fields(fields, **kwargs):
    data = {}
    for field, value in fields.items():
        if isinstance(value, dict):
            data[field] = extract_fields(value, **kwargs)
        else:
            data[field] = kwargs.get(field, None)
    return data




import os
os.environ["FALCON_API_KEY"] = "eyJraWQiOiJvMU5OWE1jNmtOS29qN1BoekV4NjZCQUZSWFZZblRCbllQVkJLbldhUWg4PSIsImFsZyI6IlJTMjU2In0.eyJzdWIiOiJhZjZmNDk5ZS1kYTMyLTRjODYtOTE0ZC1hMjRkNWJhOTE2NTgiLCJpc3MiOiJodHRwczpcL1wvY29nbml0by1pZHAudXMtZWFzdC0xLmFtYXpvbmF3cy5jb21cL3VzLWVhc3QtMV9WMjhONTZtbmciLCJjbGllbnRfaWQiOiIyaHNoamdobWxxMnVmM2gyMGtoNmd2dmx0aCIsIm9yaWdpbl9qdGkiOiIwOTM5Y2E5Mi0yMzIxLTRlZDYtODlhZC0yZDQ5ZGQ0OWYzYWUiLCJldmVudF9pZCI6ImViMzkyYjcyLTBkZmItNDBhYi04NzUxLTY0NDEzZGVkMTdmMyIsInRva2VuX3VzZSI6ImFjY2VzcyIsInNjb3BlIjoiYXdzLmNvZ25pdG8uc2lnbmluLnVzZXIuYWRtaW4iLCJhdXRoX3RpbWUiOjE2OTg4NTk2NjIsImV4cCI6MTY5ODg2MzI2MiwiaWF0IjoxNjk4ODU5NjYyLCJqdGkiOiI2OTA4ZTAwMy0zOTM3LTQzNzYtYThjYi05OWZhMDcxOTc2YTEiLCJ1c2VybmFtZSI6ImFmNmY0OTllLWRhMzItNGM4Ni05MTRkLWEyNGQ1YmE5MTY1OCJ9.tinmlf7hGT5nBuH0KKulw7ukhX46kNJRnykpP-fJs8fzUqbCGy2RVyASH2n3ib3aRXSLLY1EjnaEeFvLBUx9lowRiuRD1ivQQoxjhrIvGaxhAAavnl96Yl2BZqjXlPWawW8rozsr0GmmJLGVeLfc88B1f9qTB0YopRA5m5oUCJzB-x-g0DANyB5GFMS5n278FO9YV-HHIlj36uw1d47mvtfgYSgrZITxOcTQHLsftOdoeV9xEeMFZipwE7xHcCOpClBOgu5BGXNuimaMaGrKJTcHr8nZ6aKP4l-octUEYxCBSirEpJZYlQLdmCb6rX_qywxV8mnt7XbYNrBYl46mlQ"

from unittest import TestCase
from unittest.mock import patch, Mock
import falconai

class TestChat(TestCase): 

    # @patch("falconai.core.requests.requests.post")
    async def test_chat_response(self):
        # Mocking the response from the chat API
        # mock_response = Mock()
        # mock_response.json.return_value = {"response": "Test response"}
        # mock_post.return_value = mock_response
        
        
        response = await falconai.models.chat(
            query="Write a story about Pakistan",
            model_name="Falcon40B",
            app_type="web",
            params={
                "temperature": 1,
                "stop": ["\n\n"],
                "max_new_tokens": 900,
                "top_p": 1
            },
            context=""
        )
        
        self.assertEqual(response, {"response": "Test response"})



import asyncio
import unittest
import uuid
import os
from falconai import models_get, model_post, model_patch, model_delete


class TestModelLifecycle(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        # Generate a unique model name for use in all tests
        cls.model_name = f"test_{uuid.uuid4()}"
        cls.external_uid = None
        os.environ["FALCON_API_KEY"] = "eyJraWQiOiJvMU5OWE1jNmtOS29qN1BoekV4NjZCQUZSWFZZblRCbllQVkJLbldhUWg4PSIsImFsZyI6IlJTMjU2In0.eyJzdWIiOiJhZjZmNDk5ZS1kYTMyLTRjODYtOTE0ZC1hMjRkNWJhOTE2NTgiLCJpc3MiOiJodHRwczpcL1wvY29nbml0by1pZHAudXMtZWFzdC0xLmFtYXpvbmF3cy5jb21cL3VzLWVhc3QtMV9WMjhONTZtbmciLCJjbGllbnRfaWQiOiIyaHNoamdobWxxMnVmM2gyMGtoNmd2dmx0aCIsIm9yaWdpbl9qdGkiOiIwOTM5Y2E5Mi0yMzIxLTRlZDYtODlhZC0yZDQ5ZGQ0OWYzYWUiLCJldmVudF9pZCI6ImViMzkyYjcyLTBkZmItNDBhYi04NzUxLTY0NDEzZGVkMTdmMyIsInRva2VuX3VzZSI6ImFjY2VzcyIsInNjb3BlIjoiYXdzLmNvZ25pdG8uc2lnbmluLnVzZXIuYWRtaW4iLCJhdXRoX3RpbWUiOjE2OTg4NTk2NjIsImV4cCI6MTY5ODg2MzI2MiwiaWF0IjoxNjk4ODU5NjYyLCJqdGkiOiI2OTA4ZTAwMy0zOTM3LTQzNzYtYThjYi05OWZhMDcxOTc2YTEiLCJ1c2VybmFtZSI6ImFmNmY0OTllLWRhMzItNGM4Ni05MTRkLWEyNGQ1YmE5MTY1OCJ9.tinmlf7hGT5nBuH0KKulw7ukhX46kNJRnykpP-fJs8fzUqbCGy2RVyASH2n3ib3aRXSLLY1EjnaEeFvLBUx9lowRiuRD1ivQQoxjhrIvGaxhAAavnl96Yl2BZqjXlPWawW8rozsr0GmmJLGVeLfc88B1f9qTB0YopRA5m5oUCJzB-x-g0DANyB5GFMS5n278FO9YV-HHIlj36uw1d47mvtfgYSgrZITxOcTQHLsftOdoeV9xEeMFZipwE7xHcCOpClBOgu5BGXNuimaMaGrKJTcHr8nZ6aKP4l-octUEYxCBSirEpJZYlQLdmCb6rX_qywxV8mnt7XbYNrBYl46mlQ"

        

    def setUp(self):
        # # Create model and set external_uid for each test
        # create_response = asyncio.run(model_post(
        #     model_name=self.model_name,
        #     # ... other parameters
        # ))
        # self.external_uid = create_response.get('data', {}).get('external_uid')
        pass

    def tearDown(self):
        # # Delete model after each test
        # if self.external_uid:
        #     asyncio.run(model_delete(external_uid=self.external_uid))
        pass
    
    def run_async_test(self, coro):
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        result = loop.run_until_complete(coro)
        loop.close()
        return result


    def test_1_model_create_and_confirm(self):
        async def run_test():
            # Step 1: Create the model
            create_response = await model_post(
                model_name=self.model_name,
                model_type="base",
                version="1.0",
                year=2021,
                dataset_size="large",
                provider="provider_name",
                modality="text",
                max_tokens="512",
                description="Test model",
                model_attributes="attributes"
            )
            self.assertEqual(create_response.get('status_code'), 200, "Failed to create model")
            create_data = create_response.get('data', {})
            self.assertIn('external_uid', create_data, "Response data does not contain 'external_uid'")
            self.external_uid = create_data['external_uid']
            TestModelLifecycle.external_uid = create_data['external_uid']


            # Step 2: Confirm creation using models_get
            list_response = await models_get()
            self.assertIn(self.external_uid, [model.get('external_uid') for model in list_response.get('data', [])], 
                        f"Model with UID '{self.external_uid}' not found in models list")
         
        self.run_async_test(run_test())


    def test_2_model_patch_and_confirm(self):
        async def run_test():
            # Step 3: Patch (update) the model with new values
            patch_data = {
                'version': '2.0',
                'provider': 'updated_provider'
            }
            patch_response = await model_patch(
                id=TestModelLifecycle.external_uid,
                **patch_data
            )
            self.assertEqual(patch_response.get('status_code'), 200, "Failed to patch model")

            # Step 4: Confirm update by retrieving the updated model
            list_response = await models_get()
            updated_model = next((model for model in list_response.get('data', []) 
                                if model.get('external_uid') == TestModelLifecycle.external_uid), None)
            self.assertIsNotNone(updated_model, "Updated model not found")

            # Check if the fields were updated correctly
            for key, value in patch_data.items():
                self.assertEqual(updated_model.get(key), value, f"Field {key} was not updated correctly")
        
        self.run_async_test(run_test())



    def test_3_model_delete_and_confirm(self):
        async def run_test():
            # Step 5: Delete the model
            delete_response = await model_delete(id=TestModelLifecycle.external_uid)
            self.assertEqual(delete_response.get('status_code'), 200, "Failed to delete model")

            # Step 6: Confirm deletion using models_get
            list_response = await models_get()
            self.assertNotIn(TestModelLifecycle.external_uid, [model['external_uid'] for model in list_response.get('data', [])], 
                            "Model was not deleted")
        
        self.run_async_test(run_test())

if __name__ == '__main__':
    unittest.main()
